diff --git a/doc/src/sgml/ref/psql-ref.sgml b/doc/src/sgml/ref/psql-ref.sgml
index cc7d797..ca8ffb0 100644
--- a/doc/src/sgml/ref/psql-ref.sgml
+++ b/doc/src/sgml/ref/psql-ref.sgml
@@ -1029,11 +1029,128 @@ INSERT INTO tbl1 VALUES ($1, $2) \bind 'first value' 'second value' \g
       </varlistentry>
 
       <varlistentry id="app-psql-meta-command-conninfo">
-        <term><literal>\conninfo</literal></term>
+        <term><literal>\conninfo[+]</literal></term>
         <listitem>
         <para>
         Outputs information about the current database connection.
+        When no <literal>+</literal> is appended, it simply prints
+        a textual (string) description of a few connection options.
+        When <literal>+</literal> is given, more complete information
+        is displayed as a table containing the following columns:
         </para>
+
+        <para>
+        <literal>Database:</literal> Displays the name of the current
+        database on this connection.
+        </para>
+
+        <para>
+        <literal>Authenticated User:</literal> Displays the authenticated
+        user at the time of psql connection with the server.
+        </para>
+
+        <para>
+        <literal>System User:</literal> Returns the authentication method
+        and the identity (if any) that the user presented during the
+        authentication cycle before they were assigned a database role. It
+        is represented as auth_method:identity or NULL if the user has not
+        been authenticated.
+        </para>
+
+        <para>
+        <literal>Current User:</literal> Displays the user name of the
+        current execution context.
+        </para>
+
+        <para>
+        <literal>Session User:</literal> Displays the session user's name.
+        </para>
+
+        <para>
+        <literal>Application Name:</literal> Displays the application name,
+        in the case user sets the 'application_name' within a connection or in
+        the connection string.
+        </para>
+
+        <para>
+        <literal>Backend PID:</literal> Displays the process ID of the server
+        process attached to the current session.
+        </para>
+
+        <para>
+        <literal>Server Address:</literal> Displays the IP address on which
+        the server accepted the current connection, or NULL if the current
+        connection is via a Unix-domain socket.
+        </para>
+
+        <para>
+        <literal>Server Port:</literal> Displays the IP port number on which
+        the server accepted the current connection, or NULL if the current
+        connection is via a Unix-domain socket.
+        </para>
+
+        <para>
+        <literal>Client Address:</literal> Displays the IP address of the
+        current client, or NULL if the current connection is via a Unix-domain socket.
+        </para>
+
+        <para>
+        <literal>Client Port:</literal> Displays the IP port number of the
+        current client, or NULL if the current connection is via a Unix-domain socket.
+        </para>
+
+        <para>
+        <literal>Socket Directory:</literal> Displays the directory where
+        Unix-domain socket was created.
+        </para>
+
+        <para>
+        <literal>Host:</literal> Displays the host of the connection,
+        which refers to the address or name of the machine where the server
+        is running. When a socket connection is established, it displays NULL.
+        </para>
+
+        <para>
+        <literal>GSSAPI Authenticated:</literal> Displays "true" if GSSAPI is in use, or "false"
+        if GSSAPI is not in use on this connection.
+        </para>
+
+        <para>
+        <literal>GSSAPI Principal:</literal> Displays "principal" used to authenticate this
+        connection, or NULL if GSSAPI was not used to authenticate this connection. This field
+        is truncated if the principal is longer than NAMEDATALEN (64 characters in a standard build).
+        </para>
+
+        <para>
+        <literal>GSSAPI Encrypted:</literal> Displays "true" if GSSAPI encryption is in use on
+        this connection, or "false" if encryption is not in use.
+        </para>
+
+        <para>
+        <literal>GSSAPI Credentials Delegated:</literal> Displays "true" if GSSAPI credentials
+        were delegated on this connection, or "false" if were not delegated.
+        </para>
+
+        <para>
+        <literal>SSL Connection:</literal> Displays "true" if the current
+        connection to the server uses SSL, and "false" otherwise.
+        </para>
+
+        <para>
+        <literal>SSL Protocol:</literal> Displays the name of the protocol
+        used for the SSL connection (e.g., TLSv1.0, TLSv1.1, TLSv1.2 or TLSv1.3).
+        </para>
+
+        <para>
+        <literal>SSL Cipher:</literal> Displays the name of the cipher used
+        for the SSL connection (e.g., DHE-RSA-AES256-SHA).
+        </para>
+
+        <para>
+        <literal>SLL Compression:</literal> Displays "on" if SSL compression
+        is in use, "off" if not, or NULL if SSL is not in use on this connection.
+        </para>
+
         </listitem>
       </varlistentry>
 
diff --git a/src/bin/psql/command.c b/src/bin/psql/command.c
index 9b0fa04..fb5f63d 100644
--- a/src/bin/psql/command.c
+++ b/src/bin/psql/command.c
@@ -68,7 +68,8 @@ static backslashResult exec_command_C(PsqlScanState scan_state, bool active_bran
 static backslashResult exec_command_connect(PsqlScanState scan_state, bool active_branch);
 static backslashResult exec_command_cd(PsqlScanState scan_state, bool active_branch,
 									   const char *cmd);
-static backslashResult exec_command_conninfo(PsqlScanState scan_state, bool active_branch);
+static backslashResult exec_command_conninfo(PsqlScanState scan_state, bool active_branch,
+											 const char *cmd);
 static backslashResult exec_command_copy(PsqlScanState scan_state, bool active_branch);
 static backslashResult exec_command_copyright(PsqlScanState scan_state, bool active_branch);
 static backslashResult exec_command_crosstabview(PsqlScanState scan_state, bool active_branch);
@@ -317,8 +318,8 @@ exec_command(const char *cmd,
 		status = exec_command_connect(scan_state, active_branch);
 	else if (strcmp(cmd, "cd") == 0)
 		status = exec_command_cd(scan_state, active_branch, cmd);
-	else if (strcmp(cmd, "conninfo") == 0)
-		status = exec_command_conninfo(scan_state, active_branch);
+	else if (strcmp(cmd, "conninfo") == 0 || strcmp(cmd, "conninfo+") == 0)
+		status = exec_command_conninfo(scan_state, active_branch, cmd);
 	else if (pg_strcasecmp(cmd, "copy") == 0)
 		status = exec_command_copy(scan_state, active_branch);
 	else if (strcmp(cmd, "copyright") == 0)
@@ -643,47 +644,80 @@ exec_command_cd(PsqlScanState scan_state, bool active_branch, const char *cmd)
 }
 
 /*
- * \conninfo -- display information about the current connection
+ * \conninfo, \conninfo+ -- display information about the current connection
  */
 static backslashResult
-exec_command_conninfo(PsqlScanState scan_state, bool active_branch)
+exec_command_conninfo(PsqlScanState scan_state, bool active_branch, const char *cmd)
 {
+	bool		success = true;
+
 	if (active_branch)
 	{
 		char	   *db = PQdb(pset.db);
+		bool		show_verbose;
+
+		show_verbose = strchr(cmd, '+') ? true : false;
 
-		if (db == NULL)
-			printf(_("You are currently not connected to a database.\n"));
+		/*
+		 * \conninfo+
+		 */
+		if (show_verbose)
+			success = listConnectionInformation();
+
+		/*
+		 * \conninfo
+		 */
 		else
 		{
-			char	   *host = PQhost(pset.db);
-			char	   *hostaddr = PQhostaddr(pset.db);
+			PGresult   *res;
+			PQExpBufferData buf;
 
-			if (is_unixsock_path(host))
-			{
-				/* hostaddr overrides host */
-				if (hostaddr && *hostaddr)
-					printf(_("You are connected to database \"%s\" as user \"%s\" on address \"%s\" at port \"%s\".\n"),
-						   db, PQuser(pset.db), hostaddr, PQport(pset.db));
-				else
-					printf(_("You are connected to database \"%s\" as user \"%s\" via socket in \"%s\" at port \"%s\".\n"),
-						   db, PQuser(pset.db), host, PQport(pset.db));
-			}
+			initPQExpBuffer(&buf);
+
+			printfPQExpBuffer(&buf,
+							  "SELECT\n"
+							  "  inet_server_addr();");
+
+			res = PSQLexec(buf.data);
+
+			termPQExpBuffer(&buf);
+
+			if (!res)
+				return PSQL_CMD_ERROR;
 			else
 			{
-				if (hostaddr && *hostaddr && strcmp(host, hostaddr) != 0)
-					printf(_("You are connected to database \"%s\" as user \"%s\" on host \"%s\" (address \"%s\") at port \"%s\".\n"),
-						   db, PQuser(pset.db), host, hostaddr, PQport(pset.db));
+				char	   *host = PQhost(pset.db);
+				char	   *hostaddr = PQhostaddr(pset.db);
+
+				if (is_unixsock_path(host))
+				{
+					/* hostaddr overrides host */
+					if (hostaddr && *hostaddr)
+						printf(_("You are connected to database \"%s\" as user \"%s\" on address \"%s\" at port \"%s\".\n"),
+							   db, PQuser(pset.db), PQgetvalue(res, 0, 0), PQport(pset.db));
+					else
+						printf(_("You are connected to database \"%s\" as user \"%s\" via socket in \"%s\" at port \"%s\".\n"),
+							   db, PQuser(pset.db), host, PQport(pset.db));
+				}
 				else
-					printf(_("You are connected to database \"%s\" as user \"%s\" on host \"%s\" at port \"%s\".\n"),
-						   db, PQuser(pset.db), host, PQport(pset.db));
+				{
+					if (hostaddr && *hostaddr && strcmp(host, hostaddr) != 0)
+						printf(_("You are connected to database \"%s\" as user \"%s\" on host \"%s\" (address \"%s\") at port \"%s\".\n"),
+							   db, PQuser(pset.db), host, PQgetvalue(res, 0, 0), PQport(pset.db));
+					else
+						printf(_("You are connected to database \"%s\" as user \"%s\" on host \"%s\" at port \"%s\".\n"),
+							   db, PQuser(pset.db), host, PQport(pset.db));
+				}
+				PQclear(res);
+				printSSLInfo();
+				printGSSInfo();
 			}
-			printSSLInfo();
-			printGSSInfo();
 		}
 	}
+	else
+		ignore_slash_options(scan_state);
 
-	return PSQL_CMD_SKIP_LINE;
+	return success ? PSQL_CMD_SKIP_LINE : PSQL_CMD_ERROR;
 }
 
 /*
diff --git a/src/bin/psql/describe.c b/src/bin/psql/describe.c
index 6433497..110ab63 100644
--- a/src/bin/psql/describe.c
+++ b/src/bin/psql/describe.c
@@ -26,6 +26,7 @@
 #include "fe_utils/mbprint.h"
 #include "fe_utils/print.h"
 #include "fe_utils/string_utils.h"
+#include "libpq/pqcomm.h"
 #include "settings.h"
 #include "variables.h"
 
@@ -901,6 +902,134 @@ error_return:
 	return false;
 }
 
+/*
+ * listConnectionInformation
+ *
+ * for \conninfo+
+ */
+bool
+listConnectionInformation(void)
+{
+	PGresult   *res;
+	PQExpBufferData buf;
+	printQueryOpt myopt = pset.popt;
+
+	char	   *host = PQhost(pset.db);
+	char	   *hostaddr = PQhostaddr(pset.db);
+
+	initPQExpBuffer(&buf);
+
+	printfPQExpBuffer(&buf,
+					  "SELECT\n"
+					  "  pg_catalog.current_database() AS \"Database\",\n"
+					  "  '%s' AS \"Authenticated User\",\n",
+					  PQuser(pset.db));
+	if (pset.sversion >= 160000)
+		appendPQExpBuffer(&buf,
+						  "  pg_catalog.system_user() AS \"System User\",\n");
+	else
+		appendPQExpBuffer(&buf,
+						  "  NULL AS \"System User\",\n");
+	appendPQExpBuffer(&buf,
+					  "  pg_catalog.current_user() AS \"Current User\",\n"
+					  "  pg_catalog.session_user() AS \"Session User\",\n"
+					  "  pg_catalog.current_setting('application_name') AS \"Application Name\",\n"
+					  "  pg_catalog.pg_backend_pid() AS \"Backend PID\",\n"
+					  "  pg_catalog.inet_server_addr() AS \"Server Address\",\n"
+					  "  pg_catalog.current_setting('port') AS \"Server Port\",\n"
+					  "  pg_catalog.inet_client_addr() AS \"Client Address\",\n"
+					  "  pg_catalog.inet_client_port() AS \"Client Port\",\n");
+	if (is_unixsock_path(host) && !(hostaddr && *hostaddr))
+		appendPQExpBuffer(&buf,
+						  "  '%s' AS \"Socket Directory\",\n",
+						  host);
+	else
+		appendPQExpBuffer(&buf,
+						  "  NULL AS \"Socket Directory\",\n");
+	appendPQExpBuffer(&buf,
+					  "  CASE\n"
+					  "    WHEN\n"
+					  "      pg_catalog.inet_server_addr() IS NULL\n"
+					  "      AND pg_catalog.inet_client_addr() IS NULL\n"
+					  "    THEN NULL\n"
+					  "    ELSE '%s'\n"
+					  "  END AS \"Host\",\n",
+					  host);
+	if (pset.sversion >= 90500)
+	{
+		if (pset.sversion < 140000)
+			appendPQExpBuffer(&buf,
+							  "  ssl.ssl AS \"SSL Connection\",\n"
+							  "  ssl.version AS \"SSL Protocol\",\n"
+							  "  ssl.cipher AS \"SSL Cipher\",\n"
+							  "  ssl.compression AS \"SSL Compression\",\n");
+		if (pset.sversion >= 140000)
+			appendPQExpBuffer(&buf,
+							  "  ssl.ssl AS \"SSL Connection\",\n"
+							  "  ssl.version AS \"SSL Protocol\",\n"
+							  "  ssl.cipher AS \"SSL Cipher\",\n"
+							  "  NULL AS \"SSL Compression\",\n");
+	}
+	else
+		appendPQExpBuffer(&buf,
+						  "  NULL AS \"SSL Connection\",\n"
+						  "  NULL AS \"SSL Protocol\",\n"
+						  "  NULL AS \"SSL Cipher\",\n"
+						  "  NULL AS \"SSL Compression\",\n");
+	if (pset.sversion >= 120000)
+	{
+		if (pset.sversion >= 160000)
+			appendPQExpBuffer(&buf,
+							  "  gssapi.gss_authenticated AS \"GSSAPI Authenticated\",\n"
+							  "  gssapi.principal AS \"GSSAPI Principal\",\n"
+							  "  gssapi.\"encrypted\" AS \"GSSAPI Encrypted\",\n"
+							  "  gssapi.credentials_delegated AS \"GSSAPI Credentials Delegated\"\n");
+		if (pset.sversion < 160000)
+			appendPQExpBuffer(&buf,
+							  "  gssapi.gss_authenticated AS \"GSSAPI Authenticated\",\n"
+							  "  gssapi.principal AS \"GSSAPI Principal\",\n"
+							  "  gssapi.\"encrypted\" AS \"GSSAPI Encrypted\",\n"
+							  "  NULL AS \"GSSAPI Credentials Delegated\"\n");
+	}
+	else
+		appendPQExpBuffer(&buf,
+						  "  NULL AS \"GSSAPI Authenticated\",\n"
+						  "  NULL AS \"GSSAPI Principal\",\n"
+						  "  NULL AS \"GSSAPI Encrypted\",\n"
+						  "  NULL AS \"GSSAPI Credentials Delegated\"\n");
+	if (pset.sversion >= 90500 && pset.sversion < 120000)
+		appendPQExpBuffer(&buf,
+						  "FROM\n"
+						  "  pg_catalog.pg_stat_ssl ssl\n"
+						  "WHERE\n"
+						  "  ssl.pid = pg_catalog.pg_backend_pid()\n"
+						  ";");
+	if (pset.sversion >= 120000)
+		appendPQExpBuffer(&buf,
+						  "FROM\n"
+						  "  pg_catalog.pg_stat_ssl ssl\n"
+						  "LEFT JOIN\n"
+						  "  pg_catalog.pg_stat_gssapi gssapi ON ssl.pid = gssapi.pid\n"
+						  "WHERE\n"
+						  "  ssl.pid = pg_catalog.pg_backend_pid()\n"
+						  ";");
+	if (pset.sversion < 90500)
+		appendPQExpBuffer(&buf,
+						  ";");
+
+	res = PSQLexec(buf.data);
+	termPQExpBuffer(&buf);
+	if (!res)
+		return false;
+
+	myopt.title = _("Current Connection Information");
+	myopt.translate_header = true;
+
+	printQuery(res, &myopt, pset.queryFout, false, pset.logfile);
+
+	PQclear(res);
+	return true;
+}
 
 /*
  * listAllDbs
diff --git a/src/bin/psql/describe.h b/src/bin/psql/describe.h
index 273f974..8a4e83f 100644
--- a/src/bin/psql/describe.h
+++ b/src/bin/psql/describe.h
@@ -64,6 +64,9 @@ extern bool listTSDictionaries(const char *pattern, bool verbose);
 /* \dFt */
 extern bool listTSTemplates(const char *pattern, bool verbose);
 
+/* \conninfo */
+extern bool listConnectionInformation(void);
+
 /* \l */
 extern bool listAllDbs(const char *pattern, bool verbose);
 
diff --git a/src/bin/psql/help.c b/src/bin/psql/help.c
index 4e79a81..2c5426d 100644
--- a/src/bin/psql/help.c
+++ b/src/bin/psql/help.c
@@ -310,7 +310,7 @@ slashUsage(unsigned short int pager)
 	else
 		HELP0("  \\c[onnect] {[DBNAME|- USER|- HOST|- PORT|-] | conninfo}\n"
 			  "                         connect to new database (currently no connection)\n");
-	HELP0("  \\conninfo              display information about current connection\n");
+	HELP0("  \\conninfo[+]           display information about current connection\n");
 	HELP0("  \\encoding [ENCODING]   show or set client encoding\n");
 	HELP0("  \\password [USERNAME]   securely change the password for a user\n");
 	HELP0("\n");
